from RPi import GPIO
from time import sleep
import time
import os
from threading import Timer

class RepeatingTimer(object):

    def __init__(self, interval, f, *args, **kwargs):
        self.interval = interval
        self.f = f
        self.args = args
        self.kwargs = kwargs

        self.timer = None

    def callback(self):
        self.f(*self.args, **self.kwargs)
        self.start()

    def cancel(self):
        self.timer.cancel()

    def start(self):
        self.timer = Timer(self.interval, self.callback)
        self.timer.start()



class Potentiometer:

    #


    def __init__(self, adcnum, clockpin, mosipin, misopin, cspin, callback_client):
        # Input parameters
        #   - client function to call when switch is activated
        #
        
        self.adcnum = adcnum
        self.clockpin = clockpin
        self.mosipin = mosipin
        self.misopin = misopin
        self.cspin = cspin
        
        self.changed = False
        self.lastValue = 0
        self.tolerance = 30
        
        # set up the SPI interface pins
        GPIO.setup(self.mosipin, GPIO.OUT)
        GPIO.setup(self.misopin, GPIO.IN)
        GPIO.setup(self.clockpin, GPIO.OUT)
        GPIO.setup(self.cspin, GPIO.OUT)   
             
        self.callback_client = callback_client
        
        self.potTimer = RepeatingTimer(0.5, self.turnEventHandler)
        self.potTimer.start()
        self.potTimer.cancel()
 
    def turnEventHandler(self):
        #
        #  Read value      
        value = self.read()
        delta = abs(value - self.lastValue)
        if (delta > self.tolerance):
            self.lastValue = value
            volume = int(5 * round(value / 10.23 / 5))
            print ("Volume = " + str(volume))
            self.callback_client(volume)

    def start(self):
        self.potTimer.start()        
    

    def stop(self):
        self.potTimer.cancel()            
        
    def read(self):
        #
        # Read last value from MCP3008 ADC
        # read SPI data from MCP3008 chip, 8 possible adc's (0 thru 7)
        #def readadc(adcnum, clockpin, mosipin, misopin, cspin):
        GPIO.output(self.cspin, True)
 
        GPIO.output(self.clockpin, False)  # start clock low
        GPIO.output(self.cspin, False)     # bring CS low
        
        commandout = self.adcnum
        commandout |= 0x18  # start bit + single-ended bit
        commandout <<= 3    # we only need to send 5 bits here
        for i in range(5):
                if (commandout & 0x80):
                        GPIO.output(self.mosipin, True)
                else:
                        GPIO.output(self.mosipin, False)
                commandout <<= 1
                GPIO.output(self.clockpin, True)
                GPIO.output(self.clockpin, False)
 
        adcout = 0
        # read in one empty bit, one null bit and 10 ADC bits
        for i in range(12):
                GPIO.output(self.clockpin, True)
                GPIO.output(self.clockpin, False)
                adcout <<= 1
                if (GPIO.input(self.misopin)):
                        adcout |= 0x1
 
        GPIO.output(self.cspin, True)
        
        adcout >>= 1       # first bit is 'null' so drop it
        return adcout
 
 
